@@ -14,20 +14,7 @@
 #*                                                                           *
 #*****************************************************************************
 #*                                                                           *
-#*      $Log: Fuser.pm,v $
-#*      Revision 1.3  2005/03/17 19:16:39  jonathan
-#*      * Updated README to reflect tested kernel versions
-#*      * Reorganised the tests
-#*      * Added Test::More as prerequisite
-#*
-#*      Revision 1.2  2001/11/07 07:09:52  gellyfish
-#*      * Fixed thinko reported by Shawn Ferris
-#*      * Added test
-#*
-#*      Revision 1.1  2001/03/05 08:13:19  gellyfish
-#*      Initial revision
-#*
-#*                                                                           *
+#*      $Id: Fuser.pm 121 2009-03-10 13:46:54Z jonathan.stowe $
 #*                                                                           *
 #*****************************************************************************
 
@@ -55,7 +42,8 @@ Linux::Fuser - Determine which processes have a file open
 This module provides information similar to the Unix command 'fuser' about
 which processes have a particular file open.  The way that this works is
 highly unlikely to work on any other OS other than Linux and even then it
-may not work on other than 2.2.* kernels.
+may not work on other than 2.2.* kernels. Some features may not work
+correctly on kernel versions older than 2.6.22
 
 It should also be borne in mind that this may not produce entirely accurate
 results unless you are running the program as the Superuser as the module
@@ -71,11 +59,11 @@ owner.
 use strict;
 
 use vars qw(
-  $VERSION
-  @ISA
-);
+            $VERSION
+            @ISA
+           );
 
-$VERSION = '1.4';
+$VERSION = '1.5';
 
 =item new
 
@@ -127,33 +115,20 @@ sub fuser
     {
         opendir FD, "/proc/$proc/fd" or next;
 
-        my @fds = map { "/proc/$proc/fd/$_" } grep /^\d+$/, readdir FD;
+        my @fds = map { ["/proc/$proc/fd/$_",$_] } grep /^\d+$/, readdir FD;
 
         closedir FD;
 
-        foreach my $fd (@fds)
+        foreach my $fd_data (@fds)
         {
+            my $fd    = $fd_data->[0];
+            my $fd_no = $fd_data->[1];
+
             if ( my @statinfo = stat $fd )
             {
                 if ( ( $dev == $statinfo[0] ) && ( $ino == $statinfo[1] ) )
                 {
-                    my $user = getpwuid( ( lstat($fd) )[4] );
-
-                    my @cmd = ('');
-
-                    if ( open CMD, "/proc/$proc/cmdline" )
-                    {
-                        chomp( @cmd = <CMD> );
-                    }
-
-                    my $procinfo = {
-                        pid  => $proc,
-                        user => $user,
-                        cmd  => \@cmd
-                    };
-
-                    bless $procinfo, 'Linux::Fuser::Procinfo';
-                    push @procinfo, $procinfo;
+                   push @procinfo,Linux::Fuser::Procinfo->new($proc, $fd_data);
                 }
             }
         }
@@ -189,6 +164,11 @@ The command line of the program that opened the file.  This actually returns
 a reference to an array containing the individual elements of the command
 line.
 
+=item filedes
+
+A Linux::Fuser::FileDescriptor object that has details of the file as
+the process has it opened - see below.
+
 =back
 
 
@@ -199,6 +179,37 @@ use Carp;
 
 use vars qw($AUTOLOAD);
 
+sub new
+{
+   my ( $class, $pid, $fd_data ) = @_;
+
+   my $fd    = $fd_data->[0];
+   my $fd_no = $fd_data->[1];
+
+   my $user = getpwuid( ( lstat($fd) )[4] );
+
+   my @cmd = ('');
+
+   if ( open CMD, "/proc/$pid/cmdline" )
+   {
+      chomp( @cmd = <CMD> );
+   }
+
+   my $filedes = Linux::Fuser::FileDescriptor->new($pid, $fd_no);
+
+   my $procinfo = {
+                   pid     => $pid,
+                   user    => $user,
+                   cmd     => \@cmd,
+                   filedes => $filedes
+                  };
+
+   bless $procinfo, $class;
+
+   return $procinfo;
+
+}
+
 sub AUTOLOAD
 {
     my ( $self, @args ) = @_;
@@ -228,6 +239,102 @@ sub AUTOLOAD
 
 1;
 
+package Linux::Fuser::FileDescriptor;
+
+=head2 Linux::Fuser::FileDescriptor
+
+This is returned by the filedes method of the Linux::Fuser::Procinfo and
+contains the information about the file descriptor that the process has the
+file open under. 
+
+The information which this is based on is only available from Linux Kernel
+version 2.6.22 onwards so will not be available on earlier kernels (except
+the 'fd'.)
+
+It has the following methods (though future versions of the Linux Kernel may
+provide different or fuller information via /proc/$pid/fdinfo):
+
+=over 2
+
+=item fd
+
+The file descriptor that this file is opened under - this will be unique
+within a process (if a file is opened more than once by a process) but not
+within the system.
+
+=item flags
+
+The flags with which the file was opened (by open or creat) as a long integer.
+
+=item pos
+
+The location (in bytes) of the file pointer within the file.
+
+=back
+
+=cut
+
+use strict;
+use warnings;
+use Carp;
+
+use vars qw($AUTOLOAD);
+
+sub new
+{
+   my ( $class, $pid, $fd_no ) = @_;
+
+
+   my $self = {
+                fd => $fd_no
+              };
+
+   if ( open FDINFO,'<',"/proc/$pid/fdinfo/$fd_no" )
+   {
+      while(my $fd_info = <FDINFO>)
+      {
+         chomp($fd_info);
+         my ($key, $value ) = split /:\s+/, $fd_info;
+         $self->{$key} = $value;
+      }
+   }
+   else
+   {
+      $self->{'pos'} = undef;
+      $self->{'flags'} = undef;
+   }
+
+   return bless $self, $class;
+}
+
+sub AUTOLOAD
+{
+    my ( $self, @args ) = @_;
+
+    no strict 'refs';
+
+    ( my $method = $AUTOLOAD ) =~ s/.*://;
+
+    return if $method eq 'DESTROY';
+
+    if ( exists $self->{$method} )
+    {
+        *{$AUTOLOAD} = sub {
+            my ( $self, @args ) = @_;
+            return $self->{$method};
+        };
+    }
+    else
+    {
+        my $pack = ref($self);
+        croak "Can\'t find method $method via package $self";
+    }
+
+    goto &{$AUTOLOAD};
+
+}
+
+1;
 __END__
 
 =head2 EXPORT
@@ -236,7 +343,7 @@ None.
 
 =head1 AUTHOR
 
-Jonathan Stowe, E<lt>jns@gellyfish.comE<gt>
+Jonathan Stowe, E<lt>jns@gellyfish.co.ukE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
